@@ -655,8 +655,21 @@ static void sCommand_add_arg_to_command2(sCommand* self, MANAGED char* buf)
     self->mArgsRuntime[self->mArgsNumRuntime] = NULL;
 }
 
-static int expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunInfo* runinfo, sEnv* env, sObject* fun)
+static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* command, sRunInfo* runinfo, sObject* nextin, sObject* fun);
+
+static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunInfo* runinfo, sEnv* env, sObject* fun, sObject* nextin)
 {
+    /// expand key ///
+    char* key;
+    if(env->mKeyEnv) {
+        if(!expand_env_to_command(ALLOC &key, env->mKey, command, runinfo, nextin, fun)) {
+            return FALSE;
+        }
+    }
+    else {
+        key = STRDUP(env->mKey);
+    }
+
     switch(TYPE(object)) {
         case T_STRING: {
             add_str_to_buf(buf, string_c_str(object));
@@ -664,7 +677,7 @@ static int expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunIn
             break;
 
         case T_VECTOR:
-            if(*env->mKey == 0) {
+            if(*key == 0) {
                 int i;
                 for(i=0; i<vector_size(object); i++) {
                     sObject* object2 = vector_item(object, i);
@@ -673,7 +686,7 @@ static int expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunIn
                 }
             }
             else {
-                int index = atoi(env->mKey);
+                int index = atoi(key);
                 if(index < 0) {
                     index += vector_size(object);
                     if(index < 0) index = 0;
@@ -690,18 +703,20 @@ static int expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunIn
             break;
 
         case T_HASH:
-            if(*env->mKey == 0) {
+            if(*key == 0) {
                 char buf[128];
                 snprintf(buf, 128, "need hash key(%s)", env->mName);
                 err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                FREE(key);
                 return FALSE;
             }
             else {
-                object = hash_item(object, env->mKey);
+                object = hash_item(object, key);
                 if(object == NULL) {
                     char buf[128];
-                    snprintf(buf, 128, "invalid key(%s)", env->mKey);
+                    snprintf(buf, 128, "invalid key(%s)", key);
                     err_msg(buf, runinfo->mSName, runinfo->mSLine);
+                    FREE(key);
                     return FALSE;
                 }
 
@@ -713,11 +728,12 @@ static int expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunIn
             char buf[128];
             snprintf(buf, 128, "can't expand variable because of type(%s)", env->mName);
             err_msg(buf, runinfo->mSName, runinfo->mSLine);
-
+            FREE(key);
             return FALSE;
             }
     }
 
+    FREE(key);
     return TRUE;
 }
 
@@ -763,7 +779,7 @@ static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* comm
                     }
                 }
                 else {
-                    if(!expand_variable(command, object, &buf, runinfo, env, fun)) {
+                    if(!expand_variable(command, object, &buf, runinfo, env, fun,nextin)) {
                         FREE(buf.mBuf);
                         return FALSE;
                     }
@@ -931,7 +947,7 @@ int block_gc_children_mark(sObject* self)
     return count;
 }
 
-void sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* key)
+void sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* key, BOOL key_env)
 {
     if(self->mEnvsNum >= self->mEnvsSize) {
         self->mEnvsSize *= 2;
@@ -941,6 +957,7 @@ void sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* key)
     self->mEnvs[self->mEnvsNum].mKind = kEnv;
     self->mEnvs[self->mEnvsNum].mName = name;
     self->mEnvs[self->mEnvsNum].mKey = key;
+    self->mEnvs[self->mEnvsNum].mKeyEnv = key_env;
     self->mEnvsNum++;
 }
 
