@@ -222,12 +222,22 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     int first = atoi(buf);
                     int second = atoi(buf2);
 
-                    if(first < 0) first += vector_size(SFD(nextin).mLines);
-                    if(first < 0) first = 0;
-                    if(second < 0) second += vector_size(SFD(nextin).mLines);
-                    if(second < 0) second = 0;
-                    if(first >= vector_size(SFD(nextin).mLines)) first = vector_size(SFD(nextin).mLines) -1;
-                    if(second >= vector_size(SFD(nextin).mLines)) second = vector_size(SFD(nextin).mLines) -1;
+                    if(first < 0) {
+                        first += vector_size(SFD(nextin).mLines);
+                        if(first < 0) first = 0;
+                    }
+                    if(second < 0) {
+                        second += vector_size(SFD(nextin).mLines);
+                        if(second < 0) second = 0;
+                    }
+                    if(first >= vector_size(SFD(nextin).mLines)) {
+                        first = vector_size(SFD(nextin).mLines) -1;
+                        if(first < 0) first = 0;
+                    }
+                    if(second >= vector_size(SFD(nextin).mLines)) {
+                        second = vector_size(SFD(nextin).mLines) -1;
+                        if(second < 0) second = 0;
+                    }
 
                     if(first < second) {
                         sObject* nextin2 = FD_NEW_STACK();
@@ -273,9 +283,14 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 else {
                     int num = atoi(arg);
 
-                    if(num < 0) num += vector_size(SFD(nextin).mLines);
-                    if(num < 0) num = 0;
-                    if(num >= vector_size(SFD(nextin).mLines)) num = vector_size(SFD(nextin).mLines) -1;
+                    if(num < 0) {
+                        num += vector_size(SFD(nextin).mLines);
+                        if(num < 0) num = 0;
+                    }
+                    if(num >= vector_size(SFD(nextin).mLines)) {
+                        num = vector_size(SFD(nextin).mLines) -1;
+                        if(num < 0) num = 0;
+                    }
 
                     sObject* nextin2 = FD_NEW_STACK();
 
@@ -302,23 +317,23 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
 typedef int (*sort_if_cancelable)(void* left, void* right, sRunInfo* runinfo, sObject* nextin, sObject* nextout);
 
-static int sort_fun(void* left, void* right, sRunInfo* runinfo, sObject* nextin, sObject* nextout)
+static int sort_fun(void* left, void* right, sRunInfo* runinfo, sObject* nextin2, sObject* nextout)
 {
     sCommand* command = runinfo->mCommand;
 
     char* left2 = left;
     char* right2 = right;
 
-    fd_clear(nextin);
-    if(!fd_write(nextin, left2)) {
+    fd_clear(nextin2);
+    if(!fd_write(nextin2, left2)) {
         return -1;
     }
-    if(!fd_write(nextin, right2)) {
+    if(!fd_write(nextin2, right2)) {
         return -1;
     }
 
     int rcode = 0;
-    if(!run(command->mBlocks[0], nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
+    if(!run(command->mBlocks[0], nextin2, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
         runinfo->mRCode = rcode;
         return -1;
     }
