@@ -1,4 +1,5 @@
 #include "config.h"
+#include "pipes/pipes.h"
 #include <errno.h>
 #include <time.h>
 #include <stdlib.h>
@@ -13,6 +14,8 @@
 #include <signal.h>
 #include <limits.h>
 #include <dirent.h>
+#include <readline/readline.h>
+#include <readline/history.h>
 
 #if defined(HAVE_CURSES_H)
 #include <curses.h>
@@ -22,7 +25,6 @@
 #include <ncurses/ncurses.h>
 #endif
 
-#include "pipes/pipes.h"
 static void str_cut(enum eKanjiCode code, char* mbs, int termsize, char* dest_mbs, int dest_byte)
 {
     if(code == kUtf8) {
@@ -150,6 +152,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             if(cursor >= vector_size(SFD(nextin).mLines)) {
                 cursor = vector_size(SFD(nextin).mLines)-1;
+                if(cursor < 0) cursor = 0;
             }
             if(cursor < scrolltop) {
                 int i = cursor;
@@ -292,6 +295,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
                 if(cursor >= vector_size(SFD(nextin).mLines)) {
                     cursor = vector_size(SFD(nextin).mLines)-1;
+                    if(cursor < 0) cursor = 0;
                 }
                 if(cursor >= n) {
                     scrolltop = n;
@@ -330,16 +334,16 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(isatty(0) == 0 || isatty(1) == 0) {
-        err_msg("selector: stdin or stdout is not a tty", runinfo->mSName, runinfo->mSLine);
+        err_msg("p: stdin or stdout is not a tty", runinfo->mSName, runinfo->mSLine);
         return FALSE;
     }
     if(tcgetpgrp(0) != getpgid(0)) {
-        err_msg("selector: not forground process group", runinfo->mSName, runinfo->mSLine);
+        err_msg("p: not forground process group", runinfo->mSName, runinfo->mSLine);
         return FALSE;
     }
     char* term_env = getenv("TERM");
     if(term_env == NULL || strcmp(term_env, "") == 0) {
-        err_msg("selector: not TERM environment variable setting", runinfo->mSName, runinfo->mSLine);
+        err_msg("p: not TERM environment variable setting", runinfo->mSName, runinfo->mSLine);
         return FALSE;
     }
 
@@ -362,8 +366,6 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
 
 
-    BOOL multiple = sCommand_option_item(command, "-multiple");
-
     if(runinfo->mFilter) {
         sObject* nextin2 = FD_NEW_STACK();
 
@@ -416,12 +418,13 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             cursor = 0;
         }
 
-        if(cursor >= vector_size(SFD(nextin2).mLines)) {
-            cursor = vector_size(SFD(nextin2).mLines)-1;
-        }
         if(cursor < 0) {
             cursor = 0;
         }
+        if(cursor >= vector_size(SFD(nextin2).mLines)) {
+            cursor = vector_size(SFD(nextin2).mLines)-1;
+            if(cursor < 0) cursor = 0;
+        }
         if(cursor < scrolltop) {
             int i = cursor;
             int width_sum = 0;
@@ -511,16 +514,6 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 FREE(markfiles);
                 return FALSE;
             }
-            else if(key == 'a' && multiple) {
-                int i;
-                for(i=0; i<vector_size(SFD(nextin2).mLines); i++) {
-                    markfiles[i] = !markfiles[i];
-                }
-            }
-            else if(key == ' ' && multiple) {
-                markfiles[cursor] = !markfiles[cursor];
-                cursor++;
-            }
             else if(key == 10 || key == 13) {
                 if(!fd_write(nextout, SFD(nextin).mBuf)) {
                     err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
@@ -537,6 +530,7 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             if(cursor >= vector_size(SFD(nextin2).mLines)) {
                 cursor = vector_size(SFD(nextin2).mLines)-1;
+                if(cursor < 0) cursor = 0;
             }
             if(cursor >= n) {
                 scrolltop = n;
@@ -571,3 +565,41 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    if(isatty(0) == 0 || isatty(1) == 0) {
+        err_msg("readline: stdin or stdout is not a tty", runinfo->mSName, runinfo->mSLine);
+        return FALSE;
+    }
+    if(tcgetpgrp(0) != getpgid(0)) {
+        err_msg("readline: not forground process group", runinfo->mSName, runinfo->mSLine);
+        return FALSE;
+    }
+    char* term_env = getenv("TERM");
+    if(term_env == NULL || strcmp(term_env, "") == 0) {
+        err_msg("readline: not TERM environment variable setting", runinfo->mSName, runinfo->mSLine);
+        return FALSE;
+    }
+
+    char* prompt;
+    sCommand* command = runinfo->mCommand;
+    if(command->mArgsNumRuntime >= 2) {
+        prompt = command->mArgsRuntime[1];
+    }
+    else {
+        prompt = "> ";
+    }
+
+    char* buf = readline(prompt);
+
+    if(!fd_write(nextout, buf)) {
+        err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+        return FALSE;
+    }
+    
+    runinfo->mRCode = 0;
+
+    return TRUE;
+}
+
