@@ -355,30 +355,89 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         field = "\a";
     }
 
+    enum eKanjiCode code = gKanjiCode;
+    if(sCommand_option_item(command, "-byte")) {
+        code = kByte;
+    }
+    else if(sCommand_option_item(command, "-utf8")) {
+        code = kUtf8;
+    }
+    else if(sCommand_option_item(command, "-sjis")) {
+        code = kSjis;
+    }
+    else if(sCommand_option_item(command, "-eucjp")) {
+        code = kEucjp;
+    }
+
     /// output
     if(!runinfo->mFilter) {
         if(command->mArgsNumRuntime > 1) {
-            sObject* object = runinfo->mCurrentObject;
-            int i;
-            for(i=1; i<command->mArgsNumRuntime; i++) {
-                sObject* reciever = runinfo->mCurrentObject;
-                sObject* var = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
-                if(var && TYPE(var) == T_STRING) {
-                    if(!fd_write(nextout, string_c_str(var))) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            char* argument;
+            if(argument = sCommand_option_with_argument_item(command, "-index")) {
+                sObject* object = runinfo->mCurrentObject;
+                int i;
+                for(i=1; i<command->mArgsNumRuntime; i++) {
+                    sObject* reciever = runinfo->mCurrentObject;
+                    sObject* var = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
+
+                    if(var && TYPE(var) == T_STRING) {
+                        if(strlen(string_c_str(var)) > 0) {
+                            int len = str_kanjilen(code, string_c_str(var));
+                            int num = atoi(argument);
+                            if(num < 0) {
+                                num += len;
+                                if(num < 0) num = 0;
+                            }
+                            if(num >= len) {
+                                num = len -1;
+                                if(num < 0) num = 0;
+                            }
+
+                            char* pos = str_kanjipos2pointer(code, string_c_str(var), num);
+                            char* pos2 = str_kanjipos2pointer(code, pos, 1);
+                            if(!fd_write3(nextout, pos, pos2-pos)) {
+                                err_msg("interrupt", runinfo->mSName, runinfo->mSLine);
+                                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                                return FALSE;
+                            }
+                            if(!fd_write(nextout, field)) {
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                                return FALSE;
+                            }
+                        }
+
+                        runinfo->mRCode = 0;
+                    }
+                    else {
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
                         return FALSE;
                     }
-                    if(!fd_write(nextout, field)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                        return FALSE;
-                    }
-                    runinfo->mRCode = 0;
                 }
-                else {
-                    err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
-                    return FALSE;
+            }
+            else {
+                sObject* object = runinfo->mCurrentObject;
+                int i;
+                for(i=1; i<command->mArgsNumRuntime; i++) {
+                    sObject* reciever = runinfo->mCurrentObject;
+                    sObject* var = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
+                    if(var && TYPE(var) == T_STRING) {
+                        if(!fd_write(nextout, string_c_str(var))) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            return FALSE;
+                        }
+                        if(!fd_write(nextout, field)) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            return FALSE;
+                        }
+                        runinfo->mRCode = 0;
+                    }
+                    else {
+                        err_msg("not found variable", runinfo->mSName, runinfo->mSLine);
+                        return FALSE;
+                    }
                 }
             }
         }
@@ -745,26 +804,29 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     sObject* ary = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
 
                     if(ary && TYPE(ary) == T_VECTOR) {
-                        int num = atoi(argument);
-                        if(num < 0) {
-                            num += vector_size(ary);
-                            if(num < 0) num = 0;
-                        }
-                        if(vector_size(ary) > 0 && num >= vector_size(ary)) {
-                            num = vector_size(ary) -1;
-                        }
+                        if(vector_size(ary) > 0) {
+                            int num = atoi(argument);
+                            if(num < 0) {
+                                num += vector_size(ary);
+                                if(num < 0) num = 0;
+                            }
+                            if(num >= vector_size(ary)) {
+                                num = vector_size(ary) -1;
+                                if(num < 0) num = 0;
+                            }
 
-                        sObject* var = vector_item(ary, num);
+                            sObject* var = vector_item(ary, num);
 
-                        if(!fd_write(nextout, string_c_str(var))) {
-                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
-                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                            return FALSE;
-                        }
-                        if(!fd_write(nextout, field)) {
-                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
-                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                            return FALSE;
+                            if(!fd_write(nextout, string_c_str(var))) {
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                                return FALSE;
+                            }
+                            if(!fd_write(nextout, field)) {
+                                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
+                                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                                return FALSE;
+                            }
                         }
 
                         runinfo->mRCode = 0;
