@@ -300,7 +300,10 @@ BOOL cmd_index(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     start += len; 
                     if(start < 0) start = 0;
                 } 
-                if(len > 0 && start >= len) { start = len -1; }
+                if(start >= len) { 
+                    start = len -1; 
+                    if(start < 0) start = 0;
+                }
             }
             else {
                 start = 0;
@@ -368,6 +371,7 @@ BOOL cmd_index(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             char msg[64];
             if(result == NULL || match_count !=0) {
                 snprintf(msg, 64, "-1");
+                runinfo->mRCode = RCODE_NFUN_FALSE;
             }
             else {
                 int c = str_pointer2kanjipos(code, target, result);
@@ -504,10 +508,14 @@ BOOL cmd_rindex(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     start += len;
                     if(start < 0) start = 0;
                 }
-                if(len > 0 && start >= len) { start = len -1 ; }
+                if(start >= len) { 
+                    start = len -1 ; 
+                    if(start < 0) start = 0;
+                }
             }
             else {
                 start = len -1;
+                if(start < 0) start = 0;
             }
 
             /// get search count ///
@@ -575,6 +583,7 @@ BOOL cmd_rindex(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             char msg[64];
             if(result == NULL || match_count !=0) {
                 snprintf(msg, 64, "-1");
+                runinfo->mRCode = RCODE_NFUN_FALSE;
             }
             else {
                 int c = str_pointer2kanjipos(code, target, result);
@@ -1959,7 +1968,6 @@ BOOL cmd_add(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             number += len + 1;
             if(number < 0) number = 0;
         }
-        if(number > len) number = len;
 
         if(number < len) {
             int point = str_kanjipos2pointer(code, SFD(nextin).mBuf, number) - SFD(nextin).mBuf;
@@ -2036,7 +2044,10 @@ BOOL cmd_del(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             index += len;
             if(index < 0) index = 0;
         }
-        if(len > 0 && index >= len) index = len -1;
+        if(index >= len) {
+            index = len -1;
+            if(index < 0) index = 0;
+        }
 
         int point = str_kanjipos2pointer(code, SFD(nextin).mBuf, index) - SFD(nextin).mBuf;
         
@@ -2113,8 +2124,14 @@ BOOL cmd_rows(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         second += kanjilen;
                         if(second < 0) second = 0;
                     }
-                    if(kanjilen > 0 && first >= kanjilen) first = kanjilen -1;
-                    if(kanjilen > 0 && second >= kanjilen) second = kanjilen -1;
+                    if(first >= kanjilen) {
+                        first = kanjilen -1;
+                        if(first < 0) first = 0;
+                    }
+                    if(second >= kanjilen) {
+                        second = kanjilen -1;
+                        if(second < 0) second = 0;
+                    }
 
                     /// make table to indexing access ///
                     char** array = MALLOC(sizeof(char*)*(kanjilen+1));
@@ -2204,8 +2221,9 @@ BOOL cmd_rows(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     num += len;
                     if(num < 0) num = 0;
                 }
-                if(len > 0 && num >= len) {
+                if(num >= len) {
                     num = len -1;
+                    if(num < 0) num = 0;
                 }
 
                 sObject* nextin2 = FD_NEW_STACK();
