@@ -114,10 +114,10 @@ static BOOL read_quote(char**p, sBuf* buf, char* sname, int* sline)
 }
 
 static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObject** current_object);
+static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object);
 
 static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object)
 {
-    SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_ENV;
     (*p)++;
 
     enum eLineField lf = gLineField;
@@ -184,6 +184,8 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         sCommand_add_env_block(command, block2, lf);
     }
     else {
+        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_ENV;
+
         sBuf name;
         memset(&name, 0, sizeof(sBuf));
         name.mBuf = MALLOC(64);
@@ -226,22 +228,27 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
 
         if(**p == '[') {
             (*p)++;
-            while(1) {
-                if(**p == 0) {
-                    err_msg("close ] which is used by expand variable", sname, *sline);
-                    FREE(name.mBuf);
-                    FREE(key.mBuf);
-                    return FALSE;
-                }
-                else if(**p == ']') {
-                    (*p)++;
-                    break;
+
+            if(!read_one_argument(p, &key, sname, sline, TRUE, command, block, current_object)) 
+            {
+                FREE(name.mBuf);
+                if(key.mEnv) {  // for readline
+                    sCommand_add_arg_to_command(command, MANAGED key.mBuf, TRUE);
                 }
                 else {
-                    add_char_to_buf(&key, **p);
-                    (*p)++;
+                    FREE(key.mBuf);
                 }
+                return FALSE;
             }
+
+            skip_spaces(p);
+            if(**p != ']') {
+                err_msg("require ] character", sname, *sline);
+                FREE(name.mBuf);
+                FREE(key.mBuf);
+                return FALSE;
+            }
+            (*p)++;
         }
 
         buf->mEnv = TRUE;
@@ -252,7 +259,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         add_str_to_buf(buf, buf2);
         add_char_to_buf(buf, PARSER_MAGIC_NUMBER_ENV);
 
-        sCommand_add_env(command, MANAGED name.mBuf, MANAGED key.mBuf);
+        sCommand_add_env(command, MANAGED name.mBuf, MANAGED key.mBuf, key.mEnv);
     }
 
     return TRUE;
@@ -268,7 +275,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
         1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0 
     };
