@@ -150,9 +150,7 @@ struct sStatment;
 #define STATMENT_ANDAND 0x80
 
 #define STATMENT_COMMANDS_MAX 32
-
 #define STATMENT_OPTIONS_MAX 8
-
 #define STATMENT_OPTION_STRING_MAX 32
 
 struct block_obj;
@@ -174,6 +172,7 @@ typedef struct {
         struct {
             char* mName;
             char* mKey;
+            BOOL mKeyEnv;
         };
         struct {
             struct _sObject* mBlock;
@@ -420,6 +419,7 @@ extern char * gCommandKindStrs[kCommandMax];
 
 BOOL pipes_rehash(char* sname, int sline);
 
+BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_errmsg(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
