@@ -354,7 +354,7 @@ char** readline_on_complete(const char* text, int start, int end)
     BOOL result = parse(string_c_str(cmdline), "readline", &sline, gReadlineBlock, &gReadlineCurrentObject);
 
     /// in the block?
-    if(!result && (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_BLOCK)) {
+    if(!result && (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_BLOCK|COMPLETION_FLAGS_ENV_BLOCK)) {
         while(1) {
             if(SBLOCK(gReadlineBlock).mStatmentsNum > 0) {
                 sStatment* statment = SBLOCK(gReadlineBlock).mStatments + SBLOCK(gReadlineBlock).mStatmentsNum - 1;
@@ -413,7 +413,12 @@ char** readline_on_complete(const char* text, int start, int end)
                 ucomp_block = NULL;
             }
 
-            if(ucomp_block) {
+            if(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_ENV)) {
+                char** result = rl_completion_matches(text, env_completion);
+                stack_end_stack();
+                return result;
+            }
+            else if(ucomp_block) {
                 sObject* nextin = FD_NEW_STACK();
                 if(!fd_write(nextin, string_c_str(cmdline))) {
                     stack_end_stack();
@@ -442,11 +447,6 @@ char** readline_on_complete(const char* text, int start, int end)
                 stack_end_stack();
                 return result;
             }
-            else if(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_ENV)) {
-                char** result = rl_completion_matches(text, env_completion);
-                stack_end_stack();
-                return result;
-            }
             else if(command_num < 2 || SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT)) {
                 if(command->mMessagesNum > 0 && !(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT))) {
                     char** result = rl_completion_matches(text, message_completion);
@@ -527,6 +527,61 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+static int readline_macro(int count, int key)
+{
+    stack_start_stack();
+
+    sObject* nextout2 = FD_NEW_STACK();
+
+    int rcode = 0;
+    sObject* block = BLOCK_NEW_STACK();
+    int sline = 1;
+    if(parse("root->macro", "macro", &sline, block, NULL)) {
+        pipes_set_signal();
+
+        sObject* fun = FUN_NEW_STACK(NULL);
+        sObject* stackframe = UOBJECT_NEW_GC(8, gPipesObject, "_stackframe", FALSE);
+        vector_add(gStackFrames, stackframe);
+        //uobject_init(stackframe);
+        SFUN(fun).mLocalObjects = stackframe;
+
+        sObject* nextin2 = FD_NEW_STACK();
+
+        (void)fd_write3(nextin2, rl_line_buffer, rl_point);
+
+        if(!run(block, nextin2, nextout2, &rcode, gRootObject, fun)) {
+            if(rcode == RCODE_BREAK) {
+                fprintf(stderr, "invalid break. Not in a loop\n");
+            }
+            else if(rcode == RCODE_RETURN) {
+                fprintf(stderr, "invalid return. Not in a function\n");
+            }
+            else if(rcode == RCODE_EXIT) {
+            }
+            else {
+                fprintf(stderr, "run time error\n");
+                fprintf(stderr, "%s", string_c_str(gErrMsg));
+            }
+        }
+        pipes_restore_signal_default();
+        (void)vector_pop_back(gStackFrames);
+    }
+    else {
+        fprintf(stderr, "parser error\n");
+        fprintf(stderr, "%s", string_c_str(gErrMsg));
+    }
+
+    rl_insert_text(SFD(nextout2).mBuf);
+#if defined(__FREEBSD__)
+    mclear_immediately();
+    rl_forced_update_display();
+#endif
+
+    stack_end_stack();
+
+    return 0;
+}
+
 void pipes_readline_init(BOOL runtime_script)
 {
     rl_attempted_completion_function = readline_on_complete;
@@ -538,6 +593,8 @@ void pipes_readline_init(BOOL runtime_script)
 
     gUserCompBlocks = HASH_NEW_GC(16, FALSE);
     hash_put(gPipesObject, "_ucomp_blocks", gUserCompBlocks);
+
+    rl_bind_key('x'-'a'+1, readline_macro);
 }
 
 void pipes_readline_final()
