@@ -597,7 +597,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo)
 
 static BOOL run_exec_cprog(sCommand* command, char* program, int nextin, int nextout, int nexterr)
 {
-    /// パイプ ///
+    /// pipe ///
     if(nextin != 0) {
         if(dup2(nextin, 0) < 0) {
             char buf[32];
@@ -644,7 +644,7 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
     int nextout2;
     int nexterr2;
 
-    if(nextin == gStdin) {
+    if(nextin == gStdin && SFD(gStdin).mBufLen == 0) {
         nextin2 = 0;
     }
     else {
@@ -972,10 +972,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 {
     sObject* nextin;
     if(statment->mFlags & STATMENT_CONTEXTPIPE) {
-        if(pipein == gStdin) {
-            if(gAppType != kATOptC) {
-                fd_clear(gStdin);
-            }
+        if(pipein == gStdin && SFD(gStdin).mBufLen == 0) {
             if(!stdin_read_out()) {
                 err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
